#MicroPython SSD1306 OLED driver, I2C and SPI interfaces created by Adafruit

import time
import framebuf

# register definitions
SET_CONTRAST        = const(0x81)
SET_ENTIRE_ON       = const(0xa4)
SET_NORM_INV        = const(0xa6)
SET_DISP            = const(0xae)
SET_MEM_ADDR        = const(0x20)
SET_COL_ADDR        = const(0x21)
SET_PAGE_ADDR       = const(0x22)
SET_DISP_START_LINE = const(0x40)
SET_SEG_REMAP       = const(0xa0)
SET_MUX_RATIO       = const(0xa8)
SET_COM_OUT_DIR     = const(0xc0)
SET_DISP_OFFSET     = const(0xd3)
SET_COM_PIN_CFG     = const(0xda)
SET_DISP_CLK_DIV    = const(0xd5)
SET_PRECHARGE       = const(0xd9)
SET_VCOM_DESEL      = const(0xdb)
SET_CHARGE_PUMP     = const(0x8d)


class SSD1306:
    def __init__(self, width, height, external_vcc):
        self.width = width
        self.height = height
        self.external_vcc = external_vcc
        self.pages = self.height // 8
        # Note the subclass must initialize self.framebuf to a framebuffer.
        # This is necessary because the underlying data buffer is different
        # between I2C and SPI implementations (I2C needs an extra byte).
        self.poweron()
        self.init_display()

    def init_display(self):
        for cmd in (
            SET_DISP | 0x00, # off
            # address setting
            SET_MEM_ADDR, 0x00, # horizontal
            # resolution and layout
            SET_DISP_START_LINE | 0x00,
            SET_SEG_REMAP | 0x01, # column addr 127 mapped to SEG0
            SET_MUX_RATIO, self.height - 1,
            SET_COM_OUT_DIR | 0x08, # scan from COM[N] to COM0
            SET_DISP_OFFSET, 0x00,
            SET_COM_PIN_CFG, 0x02 if self.height == 32 else 0x12,
            # timing and driving scheme
            SET_DISP_CLK_DIV, 0x80,
            SET_PRECHARGE, 0x22 if self.external_vcc else 0xf1,
            SET_VCOM_DESEL, 0x30, # 0.83*Vcc
            # display
            SET_CONTRAST, 0xff, # maximum
            SET_ENTIRE_ON, # output follows RAM contents
            SET_NORM_INV, # not inverted
            # charge pump
            SET_CHARGE_PUMP, 0x10 if self.external_vcc else 0x14,
            SET_DISP | 0x01): # on
            self.write_cmd(cmd)
        self.fill(0)
        self.show()

    def poweroff(self):
        self.write_cmd(SET_DISP | 0x00)

    def contrast(self, contrast):
        self.write_cmd(SET_CONTRAST)
        self.write_cmd(contrast)

    def invert(self, invert):
        self.write_cmd(SET_NORM_INV | (invert & 1))

    def show(self):
        x0 = 0
        x1 = self.width - 1
        if self.width == 64:
            # displays with width of 64 pixels are shifted by 32
            x0 += 32
            x1 += 32
        self.write_cmd(SET_COL_ADDR)
        self.write_cmd(x0)
        self.write_cmd(x1)
        self.write_cmd(SET_PAGE_ADDR)
        self.write_cmd(0)
        self.write_cmd(self.pages - 1)
        self.write_framebuf()

    def fill(self, col):
        self.framebuf.fill(col)

    def pixel(self, x, y, col):
        self.framebuf.pixel(x, y, col)

    def scroll(self, dx, dy):
        self.framebuf.scroll(dx, dy)

    def text(self, string, x, y, col=1):
        self.framebuf.text(string, x, y, col)


class SSD1306_I2C(SSD1306):
    def __init__(self, width, height, i2c, addr=0x3c, external_vcc=False):
        self.i2c = i2c
        self.addr = addr
        self.temp = bytearray(2)
        # Add an extra byte to the data buffer to hold an I2C data/command byte
        # to use hardware-compatible I2C transactions.  A memoryview of the
        # buffer is used to mask this byte from the framebuffer operations
        # (without a major memory hit as memoryview doesn't copy to a separate
        # buffer).
        self.buffer = bytearray(((height // 8) * width) + 1)
        self.buffer[0] = 0x40  # Set first byte of data buffer to Co=0, D/C=1
        self.framebuf = framebuf.FrameBuffer1(memoryview(self.buffer)[1:], width, height)
        super().__init__(width, height, external_vcc)

    def write_cmd(self, cmd):
        self.temp[0] = 0x80 # Co=1, D/C#=0
        self.temp[1] = cmd
        self.i2c.writeto(self.addr, self.temp)

    def write_framebuf(self):
        # Blast out the frame buffer using a single I2C transaction to support
        # hardware I2C interfaces.
        self.i2c.writeto(self.addr, self.buffer)

    def poweron(self):
        pass


class SSD1306_SPI(SSD1306):
    def __init__(self, width, height, spi, dc, res, cs, external_vcc=False):
        self.rate = 10 * 1024 * 1024
        dc.init(dc.OUT, value=0)
        res.init(res.OUT, value=0)
        cs.init(cs.OUT, value=1)
        self.spi = spi
        self.dc = dc
        self.res = res
        self.cs = cs
        self.buffer = bytearray((height // 8) * width)
        self.framebuf = framebuf.FrameBuffer1(self.buffer, width, height)
        super().__init__(width, height, external_vcc)

    def write_cmd(self, cmd):
        self.spi.init(baudrate=self.rate, polarity=0, phase=0)
        self.cs.high()
        self.dc.low()
        self.cs.low()
        self.spi.write(bytearray([cmd]))
        self.cs.high()

    def write_framebuf(self):
        self.spi.init(baudrate=self.rate, polarity=0, phase=0)
        self.cs.high()
        self.dc.high()
        self.cs.low()
        self.spi.write(self.buffer)
        self.cs.high()

    def poweron(self):
        self.res.high()
        time.sleep_ms(1)
        self.res.low()
        time.sleep_ms(10)
        self.res.high()


# ********************************************************************************
# FileName : ex_08
# Description : 버튼을 누르면 버튼과 같은색의 LED를 켜는 예제
# Author : 이인정
# Created Date : 2021.05.31
# Reference :
# Modified : 2021.06.01 : LIJ : 헤더수정
# ********************************************************************************

# import
from ETboard.lib.pin_define import *
from machine import Pin


# setup
# Button 입력 모드 설정
Pin6 = Pin(D6, Pin.IN)       # 빨강색 버튼 입력모드 설정하기
Pin7 = Pin(D7, Pin.IN)       # 파랑색 버튼 입력모드 설정하기
Pin8 = Pin(D8, Pin.IN)       # 초록색 버튼 입력모드 설정하기
Pin9 = Pin(D9, Pin.IN)       # 노랑색 버튼 입력모드 설정하기

# LED 출력 모드 설정
Pin2 = Pin(D2, Pin.OUT)      # 빨강색 LED 출력모드 설정하기
Pin3 = Pin(D3, Pin.OUT)      # 파랑색 LED 출력모드 설정하기
Pin4 = Pin(D4, Pin.OUT)      # 초록색 LED 출력모드 설정하기
Pin5 = Pin(D5, Pin.OUT)      # 노랑색 LED 출력모드 설정하기


# main loop
while True:
    if Pin6.value() == LOW:  # 빨강 버튼이 눌리면 빨강 LED 켜기
        Pin2.value(HIGH)
    
    if Pin7.value() == LOW:  # 파랑 버튼이 눌리면 파랑 LED 켜기
        Pin3.value(HIGH)
    
    if Pin8.value() == LOW:  # 초록 버튼이 눌리면 초록 LED 켜기
        Pin4.value(HIGH)
    
    if Pin9.value() == LOW:  # 노랑 버튼이 눌리면 노랑 LED 켜기
        Pin5.value(HIGH)
    
# ┌───────────────────────────────────────────┐
# │                                           │
# │(주)한국공학기술연구원 http://et.ketri.re.kr│
# │                                           │
# └───────────────────────────────────────────┘



from machine import Pin, SoftI2C
from ETboard.lib import ssd1306
from ETboard.lib.pin_define import *
from machine import ADC


class oled_u8g2:
    def __init__(self):
        i2c = SoftI2C(scl=Pin(22), sda=Pin(21)) # oled 통신핀 기능 설정
        oled_width = 128 # oled 너비
        oled_height = 64 # oled 높이
        self.oled = ssd1306.SSD1306_I2C(oled_width, oled_height, i2c)
        
    def __del__(self):
        pass
    
    def clear(self):
        self.oled.fill(0)
    
    def setLine(self, lineNum, text):
        yNum = (lineNum - 1) * 8
        self.oled.text(text, 0, yNum)
    
    def display(self):
        self.oled.show()



# ********************************************************************************
# FileName : ex_10
# Description : 초음파센서를 이용해 거리를 측정하는 예제
#               거리 값에 따라서 LED가 점등함
# Author : 이인정
# Created Date : 2021.05.31
# Reference :
# Modified : 2021.06.01 : LIJ : 헤더수정
# ********************************************************************************

# import
from ETboard.lib.pin_define import *
from machine import Pin, time_pulse_us
import time

# global definition
trigPin = Pin(D9, Pin.OUT)  # 초음파 송신부
echoPin = Pin(D8, Pin.IN)   # 초음파 수신부
usw_threshold_1 = 10        # 초음파센서 임계치_1
usw_threshold_2 = 20        # 초음파센서 임계치_2


# setup
# LED 출력모드 설정
Pin2 = Pin(D2, Pin.OUT)     # 빨강 LED 출력 모드 설정하기
Pin4 = Pin(D4, Pin.OUT)     # 초록 LED 출력 모드 설정하기
Pin5 = Pin(D5, Pin.OUT)     # 노랑 LED 출력 모드 설정하기


# main loop
while True:
    # 초음파 송신 후 수신부는 HIGH 상태로 대기
    trigPin.value(LOW)
    echoPin.value(LOW)
    time.sleep_ms(2)
    trigPin.value(HIGH)
    time.sleep_ms(10)
    trigPin.value(LOW)
    
    # echoPin 이 HIGH를 유지한 시간 저장
    duration = time_pulse_us(echoPin, HIGH)
    # HIGH 였을 때 시간(초음파 송수신 시간)을 기준으로 거리를 계산
    distance = ((340 * duration) / 10000) / 2
    
    # 초음파센서 값을 출력
    print(distance, " cm ")                        # 거리를 화면에 출력해줌
    time.sleep_ms(100)                             # 0.1초 대기
    
    # 초음파센서 값에 따라 LED 제어
    if distance < usw_threshold_1:                 # 거리가 usw_threshold_1 미만이면
        Pin2.value(HIGH)                           # 빨강색 LED 켜짐
    else:
        Pin2.value(LOW)                            # 빨강색 LED 꺼짐

    # usw_threshold_2 초과이면 usw_threshold_1 미만이면
    if (distance < usw_threshold_2) and (distance > usw_threshold_1):
        Pin5.value(HIGH)                           # 노랑색 LED 켜짐
    else:
        Pin5.value(LOW)                            # 노랑색 LED 꺼짐
        
    if distance > usw_threshold_2:                 # usw_threshold_1 미만이면
        Pin4.value(HIGH)                           # 초록색 LED 켜짐
    else:
        Pin4.value(LOW)                            # 초록색 LED 꺼짐
        
# ┌───────────────────────────────────────────┐
# │                                           │
# │(주)한국공학기술연구원 http://et.ketri.re.kr│
# │                                           │
# └───────────────────────────────────────────┘



# ********************************************************************************
# FileName : ex_09
# Description : OLED에 문자열을 출력하는 예제
#               조도 센서의 값에 따라서 다른 문자열이 출력됨
# Author : 이인정
# Created Date : 2021.05.31
# Reference :
# Modified : 2021.06.01 : LIJ : 헤더수정
# ********************************************************************************

# import
from ETboard.lib.OLED_U8G2 import *
from machine import ADC

# global definition
oled = oled_u8g2()
CDS_threshold_1 = 300                # 조도센서 임계치_1
CDS_threshold_2 = 100                # 조도센서 임계치_2


# setup
sensor = ADC(Pin(A1))                 # 조도센서
sensor.atten(ADC.ATTN_11DB)


# main loop
while True:
    CDS_Value = sensor.read() / 16
    print(" 조도 센서 : ", CDS_Value)
    print("---------------------")

    if CDS_Value > CDS_threshold_1:
        oled.clear()                  # oled 내용을 지우기
        oled.setLine(1, "^^")         # 1번째 줄에 ^^ 출력하기
        oled.setLine(2, "GOOD")       # 2번째 줄에 GOOD 출력하기
        oled.setLine(3, "MORNING")    # 3번째 줄에 MORNING 출력하기

    if CDS_Value < CDS_threshold_2:
        oled.clear()                  # oled 내용을 지우기
        oled.setLine(1, "******")     # 1번째 줄에 ****** 출력하기
        oled.setLine(2, "GOOD")       # 2번째 줄에 GOOD 출력하기
        oled.setLine(3, "NIGHT!")     # 3번째 줄에 NIGHT! 출력하기

    oled.display()                    # 저장된 내용을 oled에 보여줌

# ┌───────────────────────────────────────────┐
# │                                           │
# │(주)한국공학기술연구원 http://et.ketri.re.kr│
# │                                           │
# └───────────────────────────────────────────┘



# ********************************************************************************
# FileName : ex_03
# Description : 4개의 LED를 순서대로 켜고 끄는 예제
# Author : 이인정
# Created Date : 2021.05.31
# Reference :
# Modified : 2021.06.01 : LIJ : 헤더수정
# ********************************************************************************

# import
from ETboard.lib.pin_define import *
from machine import Pin
import time

# global definition
count = 0                  # 4개의 LED를 1회만 켜고 끄기 위한 변수


# setup
PinD2 = Pin(D2, Pin.OUT)   # D2를 LED 출력모드 설정하기
PinD3 = Pin(D3, Pin.OUT)   # D3를 LED 출력모드 설정하기
PinD4 = Pin(D4, Pin.OUT)   # D4를 LED 출력모드 설정하기
PinD5 = Pin(D5, Pin.OUT)   # D5를 LED 출력모드 설정하기


# main loop
while count < 1:
    time.sleep(1)          # 1초 기다리기
    PinD2.value(HIGH)      # 빨강 LED 켜기
    time.sleep(1)          # 1초 기다리기
    PinD4.value(HIGH)      # 초록 LED 켜기
    time.sleep(1)          # 1초 기다리기
    PinD5.value(HIGH)      # 노랑 LED 켜기
    time.sleep(1)          # 1초 기다리기
    PinD3.value(HIGH)      # 파랑 LED 켜기
    
    time.sleep(1)          # 1초 기다리기
    PinD2.value(LOW)       # 빨강 LED 끄기
    time.sleep(1)          # 1초 기다리기
    PinD4.value(LOW)       # 초록 LED 끄기
    time.sleep(1)          # 1초 기다리기
    PinD5.value(LOW)       # 노랑 LED 끄기
    time.sleep(1)          # 1초 기다리기
    PinD3.value(LOW)       # 파랑 LED 끄기
    
    count = 1              # LED를 제어하지 않기 위해 count 변수를 1로 변경

# ┌───────────────────────────────────────────┐
# │                                           │
# │(주)한국공학기술연구원 http://et.ketri.re.kr│
# │                                           │
# └───────────────────────────────────────────┘



EXTERNAL_NUM_INTERRUPTS = 16
NUM_DIGITAL_PINS        = 40
NUM_ANALOG_INPUTS       = 7

TX = 34;
RX = 35;

SDA = 21;
SCL = 22;

SS    = 29;
MOSI  = 37;
MISO  = 31;
SCK   = 30;

A0 = 36;   # BUILTIN_Potentiometer
A1 = 39;   # BUILTIN_CDS
A2 = 32;   # BUILTIN_temperature
A3 = 33;   # Analog Input
A4 = 34;   # Analog Input
A5 = 35;   # Analog Input
A6 = 25;   # Analog Input
A7 = 26;   # Analog Input


D2 = 27;   # BUILTIN_LED_Red
D3 = 14;   # BUILTIN_LED_Blue
D4 = 12;   # BUILTIN_LED_Green
D5 = 13;   # BUILTIN_LED_Yellow
D6 = 15;   # BUILTIN_BUTTON_Red
D7 = 16;   # BUILTIN_BUTTON_Blue
D8 = 17;   # BUILTIN_BUTTON_Green
D9 = 4;    # BUILTIN_BUTTON_Yellow

DAC1 = 25;
DAC2 = 26;

HIGH = 1
LOW = 0


# ********************************************************************************
# FileName : ex_06
# Description : 4개의 LED를 빨간 버튼이
#               눌릴때까지 껏다 켰다 반복하는 예제
# Author : 이인정
# Created Date : 2021.05.31
# Reference :
# Modified : 2021.06.01 : LIJ : 헤더수정
# ********************************************************************************

# import
from ETboard.lib.pin_define import *
from machine import Pin
import time

# global definition
count = 0                    # 4개의 LED를 빨간 버튼이 눌릴때까지 껏다 켰다 반복 하기 위한 변수


# setup
PinD2 = Pin(D2, Pin.OUT)     # D2를 LED 출력모드 설정하기
PinD3 = Pin(D3, Pin.OUT)     # D3를 LED 출력모드 설정하기
PinD4 = Pin(D4, Pin.OUT)     # D4를 LED 출력모드 설정하기
PinD5 = Pin(D5, Pin.OUT)     # D5를 LED 출력모드 설정하기
PinD6 = Pin(D6, Pin.IN)      # D6을 버튼 입력모드 설정하기


# main loop
while count < 1:
    time.sleep(1)            # 1초 기다리기
    PinD2.value(HIGH)        # 빨강 LED 켜기
    PinD3.value(HIGH)        # 파란 LED 켜기
    PinD4.value(HIGH)        # 초록 LED 켜기
    PinD5.value(HIGH)        # 노랑 LED 켜기
    
    time.sleep(1)            # 1초 기다리기
    PinD2.value(LOW)         # 빨강 LED 끄기
    PinD3.value(LOW)         # 파랑 LED 끄기
    PinD4.value(LOW)         # 초록 LED 끄기
    PinD5.value(LOW)         # 노랑 LED 끄기
    
    if PinD6.value() == 0: 
        count = 1 

# ┌───────────────────────────────────────────┐
# │                                           │
# │(주)한국공학기술연구원 http://et.ketri.re.kr│
# │                                           │
# └───────────────────────────────────────────┘



# ********************************************************************************
# FileName : ex_01
# Description : 빨간 LED를 2회 켜고 끄는 예제
# Author : 이인정
# Created Date : 2021.05.31
# Reference :
# Modified : 2021.06.01 : LIJ : 헤더수정
# ********************************************************************************

# import
from ETboard.lib.pin_define import *
from machine import Pin
import time

# global definition
count = 0                  # LED를 2회만 켜고 끄기를 위한 변수


# setup
PinD2 = Pin(D2, Pin.OUT)   # D2를 LED 출력모드 설정


# main loop
while count < 1:
    time.sleep(2)          # 2초 기다리기
    PinD2.value(HIGH)      # LED 켜기
    time.sleep(2)          # 2초 기다리기
    PinD2.value(LOW)       # LED 끄기

    time.sleep(2)          # 2초 기다리기
    PinD2.value(HIGH)      # LED 켜기
    time.sleep(2)          # 2초 기다리기
    PinD2.value(LOW)       # LED 끄기

    count = 1              # 프로그램을 종료시키기 위해 count 변수값 변경

# ┌───────────────────────────────────────────┐
# │                                           │
# │(주)한국공학기술연구원 http://et.ketri.re.kr│
# │                                           │
# └───────────────────────────────────────────┘



# ********************************************************************************
# FileName : ex_07
# Description : 빨간 버튼이 눌리면 빨간 LED를 켜고
#               다시 버튼이 눌리면 빨간 LED를 끄는 예제
# Author : 이인정
# Created Date : 2021.05.31
# Reference :
# Modified : 2021.06.01 : LIJ : 헤더수정
# ********************************************************************************

# import
from ETboard.lib.pin_define import *
from machine import Pin
import time

# global definition
state = LOW                # LED 상태
reading = HIGH             # 버튼 상태
previous = LOW             # 버튼 이전 상태
_time = 0                  # LED가 ON/OFF 토글된 마지막 시간
debounce = 100             # debounce(눌린시간) 시간 설정


# setup
PinD2 = Pin(D2, Pin.OUT)   # D2를 LED 출력모드 설정하기
Pin6 = Pin(D6, Pin.IN)     # D6을 버튼 입력모드 설정하기


# main loop
while True:
    reading = Pin6.value()  # Button 상태 읽기
    
    # 버튼이 눌려졌고 버튼 눌림 시간이 debounce(설정해둔 눌린시간) 시간보다 크면 실행
    if reading == HIGH and previous == LOW and time.ticks_ms() - _time > debounce:
        if state == HIGH:
            state = LOW
        else:
            state = HIGH
        
        _time = time.ticks_ms()
    
    PinD2.value(state)
    
    previous = reading

# ┌───────────────────────────────────────────┐
# │                                           │
# │(주)한국공학기술연구원 http://et.ketri.re.kr│
# │                                           │
# └───────────────────────────────────────────┘



import tkinter as tk
import os
import time
from glob import glob
from thonny import get_workbench, workbench
from tkinter.messagebox import showinfo

class ETBoardMenu:
    def __init__(self):
        self.workbench = get_workbench()
        self.root_example_menu = tk.Menu(self.workbench, tearoff=False, postcommand = self.update_example_menu)
        
    def __del__(self):
        pass

    def hello(self):
        #핸들러 테스트 함수
        showinfo("Hello!", "Thonny rules!")

    def get_examples_list(self):
        examples_list = []
        target_path = os.path.abspath(__file__)
        target_path = os.path.dirname(target_path)
        for root, dirs_, files in os.walk(target_path):
            for file in files:
                name = file.split(os.path.sep)[-1]
                ext = file.split(".")[-1]
                full_path = os.path.join(root, file)
                dir = os.path.dirname(full_path).split(os.path.sep)[-1]
                if ext != "py" or name == "__init__.py" or dir == "lib":
                    continue
                
                examples_list.append(full_path)

        return examples_list

    def update_example_menu(self):
        #예제 파일 리스트를 가져옴, 풀 경로가 들어옴
        example_list = self.get_examples_list()

        #기존 리스트 초기화
        self.root_example_menu.delete(0, "end")

        dir_map = {}
        for path in example_list:
            name = path.split(os.path.sep)[-1]
            dir = os.path.dirname(path)

            def load(path=path):
                editor = self.workbench.get_editor_notebook()
                if editor.file_is_opened(path):
                    return

                editor.show_file(path)
            
            menu = None
            if dir in dir_map:
                menu = dir_map[dir]
            else:
                menu = tk.Menu(self.workbench, tearoff=False)
                dir_map[dir] = menu
            
            if menu is None:
                continue

            menu.add_command(label=name, command=load)
        
        for key, value in dir_map.items():
            name = key.split(os.path.sep)[-1]
            menu = value            
            self.root_example_menu.add_cascade(label=name, menu=menu)
        
    def load_plugin(self):
        self.workbench.add_command(command_id="et-examples",
                                menu_name="file",
                                command_label="ET-board 예제",
                                group=10,
                                submenu=self.root_example_menu)

        


if get_workbench() is not None:
    run = ETBoardMenu().load_plugin()
